use serenity::{
    model::{channel::ReactionType, prelude::*},
    prelude::TypeMapKey,
    prelude::{Context, EventHandler},
};

extern crate rand;
use rand::{
    distributions::{Distribution, Standard},
    Rng,
};
use std::collections::HashMap;
use std::convert::TryFrom;
use std::sync::{Arc, Mutex};
pub struct Handler;

impl EventHandler for Handler {
    fn ready(&self, ctx: Context, r: Ready) {
        println!("Successfully logged in as {}", r.user.name);
        println!("Can see {} guilds", r.guilds.len());
        let game = Activity::listening("CHEESE");
        let status = OnlineStatus::DoNotDisturb;

        ctx.set_presence(Some(game), status);
    }
    fn message(&self, ctx: Context, new_message: Message) {
        if new_message.mentions_user_id(140652945032216576) {
            new_message.react(&ctx, '🐀').ok();
        }
    }
    fn reaction_add(&self, context: Context, reaction: Reaction) {
        let reaction_clone = reaction.clone();
        if let ReactionType::Unicode(emoji) = reaction.emoji {
            match emoji.as_ref() {
                "🐱" | "🐀" | "🧀" => play_crc(reaction_clone, context, &emoji),
                _ => {}
            }
        }
    }
}

fn play_crc(reaction: Reaction, mut context: Context, emoji: &str) {
    let game_store = {
        let mut data = context.data.write();
        data.get_mut::<CRCGameStateStore>()
            .expect("Expected CRCGameStateStore")
            .clone()
    };
    let mut remove_message = false;
    if let Some(current_game) = game_store
        .lock()
        .unwrap()
        .get_mut(&u64::from(reaction.message_id))
    {
        if reaction.user_id != current_game.player_id {
            return;
        }
        let message = current_game
            .play(CRCChoices::try_from(emoji).unwrap())
            .to_string();
        let mut score_message = format!(
            "your score: {} - my score {}\n{}",
            current_game.player_score, current_game.bot_score, message
        )
        .to_string();
        if current_game.player_score == 3 {
            score_message.push_str("\n**You win!**");
            remove_message = true;
        } else if current_game.bot_score == 3 {
            score_message.push_str("\n**I win :)**");
            remove_message = true;
        }
        if let Ok(mut discord_message) = reaction.message(&mut context.http) {
            discord_message
                .edit(&context, |e| e.content(score_message))
                .ok();
        };
        reaction.delete(&context).ok();
    };
    if remove_message {
        game_store
            .lock()
            .unwrap()
            .remove(&reaction.message_id.into());
    }
}
#[derive(PartialEq)]
pub enum CRCChoices {
    Cat,
    Rat,
    Cheese,
}
impl Distribution<CRCChoices> for Standard {
    fn sample<R: Rng + ?Sized>(&self, rng: &mut R) -> CRCChoices {
        match rng.gen_range(0, 3) {
            0 => CRCChoices::Cat,
            1 => CRCChoices::Rat,
            _ => CRCChoices::Cheese,
        }
    }
}
impl TryFrom<&str> for CRCChoices {
    type Error = String;
    fn try_from(emoji: &str) -> Result<CRCChoices, Self::Error> {
        use CRCChoices::*;
        match emoji {
            "🐀" => Ok(Rat),
            "🐱" => Ok(Cat),
            "🧀" => Ok(Cheese),
            _ => Err("naughty emoji".to_string()),
        }
    }
}

pub struct CRCGameState {
    pub player_id: u64,
    pub player_score: u8,
    pub bot_score: u8,
}

impl CRCGameState {
    pub fn play(&mut self, player: CRCChoices) -> String {
        let bot = rand::random::<CRCChoices>();
        use CRCChoices::*;

        let message_tail = if bot == player {
            "draw"
        } else if player == Rat {
            if bot == Cheese {
                self.player_score += 1;
                "your rat eats the cheese and you win!"
            } else {
                self.bot_score += 1;
                "my cat eats your rat and I win"
            }
        } else if player == Cheese {
            if bot == Rat {
                self.bot_score += 1;
                "bots rat eats the cheese and you lose"
            } else {
                self.player_score += 1;
                "my cat eats your rat and I win"
            }
        } else if player == Cat {
            if bot == Rat {
                self.player_score += 1;
                "your cat eats the rat so you win..."
            } else {
                self.bot_score += 1;
                "your cat eats the cheese but it gets poisoned and dies, I win"
            }
        } else {
            "bug report plz"
        };
        format!("bot picked {} {}", bot, message_tail)
    }
}

impl std::fmt::Display for CRCChoices {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        let string_form = match self {
            CRCChoices::Rat => "🐀",
            CRCChoices::Cat => "🐱",
            CRCChoices::Cheese => "🧀",
        };
        write!(f, "{}", string_form)
    }
}

pub struct CRCGameStateStore;
impl TypeMapKey for CRCGameStateStore {
    type Value = Arc<Mutex<HashMap<u64, CRCGameState>>>;
}
