use std::collections::{HashMap, HashSet};

use serenity::{
    client::Client,
    framework::standard::{
        help_commands,
        macros::{group, help},
        Args, CommandGroup, CommandResult, HelpOptions, StandardFramework,
    },
    model::prelude::*,
    prelude::*,
};

use std::env;
use std::sync::{Arc, Mutex};
mod commands;
use commands::{
    AVATAR_COMMAND, CAT_RAT_CHEESE_COMMAND, DOG_COMMAND, ECHO_COMMAND, EIGHTBALL_COMMAND,
    PING_COMMAND, PROFILE_COMMAND, PUG_COMMAND, SPAM_COMMAND,
};
mod events;

#[help]
fn my_help(
    ctx: &mut Context,
    msg: &Message,
    args: Args,
    help_options: &'static HelpOptions,
    groups: &[&'static CommandGroup],
    owners: HashSet<UserId>,
) -> CommandResult {
    help_commands::with_embeds(ctx, msg, args, &help_options, groups, owners)
}

fn main() {
    let mut client = Client::new(&env::var("DISCORD_TOKEN").expect("token"), events::Handler)
        .expect("Error creating client");
    client.with_framework(
        StandardFramework::new()
            .configure(|c| c.prefix("&"))
            .help(&MY_HELP)
            .group(&FUN_GROUP)
            .group(&DISCORD_GROUP)
            .group(&UTILITY_GROUP)
            .group(&DOGS_GROUP)
            .group(&RUNESCAPE_GROUP),
    );
    {
        let mut data = client.data.write();
        let crc_game: HashMap<u64, events::CRCGameState> = HashMap::new();
        data.insert::<events::CRCGameStateStore>(Arc::new(Mutex::new(crc_game)));
    }
    if let Err(why) = client.start() {
        println!("An error occurred while running the client: {:?}", why);
    }
}

group!({
    name: "fun",
    options: {},
    commands: [cat_rat_cheese, echo, spam, eightball],
});

group!({
    name: "discord",
    options: {},
    commands: [avatar]
});

group!({
    name: "utility",
    options: {},
    commands: [ping]
});

group!({
    name: "dogs",
    options: {},
    commands: [pug, dog]
});

group!({
    name: "runescape",
    options: {},
    commands: [profile]
});
